import { AutoMap } from '@automapper/classes';
import {
  IsOptional,
  IsBoolean,
  IsInt,
  IsEnum,
  ValidateNested,
  ArrayMinSize,
  IsUUID,
  IsArray,
  IsISO8601,
} from 'class-validator';
import { Transform, Type } from 'class-transformer';
import { ScheduleDto } from './schedule.dto';
import { MarginDurationsDto } from './margin-durations.dto';
import { WaypointDto } from './waypoint.dto';
import { intToFrequency } from './transformers/int-to-frequency';
import { IsSchedule } from './validators/decorators/is-schedule.decorator';
import { HasValidPositionIndexes } from './validators/decorators/has-valid-position-indexes.decorator';
import { Frequency } from '@modules/ad/core/ad.types';

export class CreateAdRequestDto {
  @IsUUID(4)
  @AutoMap()
  userId: string;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  driver?: boolean;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  passenger?: boolean;

  @Transform(({ value }) => intToFrequency(value), {
    toClassOnly: true,
  })
  @IsEnum(Frequency)
  @AutoMap()
  frequency: Frequency;

  @IsISO8601({
    strict: true,
    strictSeparator: true,
  })
  @AutoMap()
  fromDate: string;

  @IsISO8601({
    strict: true,
    strictSeparator: true,
  })
  @AutoMap()
  toDate: string;

  @Type(() => ScheduleDto)
  @IsSchedule()
  @ValidateNested({ each: true })
  @AutoMap()
  schedule: ScheduleDto;

  @IsOptional()
  @Type(() => MarginDurationsDto)
  @ValidateNested({ each: true })
  @AutoMap()
  marginDurations?: MarginDurationsDto;

  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsProposed?: number;

  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsRequested?: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  strict?: boolean;

  @Type(() => WaypointDto)
  @IsArray()
  @ArrayMinSize(2)
  @HasValidPositionIndexes()
  @ValidateNested({ each: true })
  @AutoMap()
  waypoints: WaypointDto[];
}
